module Scrivito
  class BinaryRedirectController < ActionController::Base
    def to_binary
      binary = BinaryParamVerifier.verify(params[:encrypted_params])
      redirect_to BinaryRewrite.call(request, binary.url), status: 301, allow_other_host: true
    rescue BinaryParamVerifier::InvalidSignature
      head :precondition_failed
    rescue TransformationSourceError => error
      case error.code
      when /transform.source.too_large/ then render_error('source_too_large')
      else render_error('source_invalid')
      end
    end

    private

    def render_error(error_name)
      redirect_to view_context.image_path("scrivito/#{error_name}.png"), allow_other_host: true
    end
  end
end
