Rails.application.routes.draw do
  mount Scrivito::SdkEngine, at: '/', as: :scrivito_engine
end

Scrivito::SdkEngine.routes.draw do
  get 'scrivito', to: 'scrivito/ui#index'
  get 'scrivito/*application_path', to: 'scrivito/ui#index', format: false

  scope '__scrivito', module: 'scrivito' do
    get ':id', to: 'cms_dispatch#index', as: :base_id, constraints: {id: /\h{16}/}

    resources :objs, controller: 'objs', only: [], defaults: {format: :json} do
      collection do
        get :page_class_selection
        get :valid_page_classes
        get :search
      end

      member do
        get :widget
        get :widget_class_selection
        get :widget_modification
      end
    end

    resources :sessions, controller: 'sessions', only: [:update], defaults: {format: :json}

    get 'obj_class/:obj_class_name/defaults', to: 'obj_class#defaults'
    get 'obj_class/thumbnails', to: 'obj_class#thumbnails'

    put 'resolve_paths' => 'resolve_paths#resolve', defaults: {format: :json}

    get 'workspaces/:id/check', to: 'workspaces#check', defaults: {format: :json}
    put 'workspaces/:id/publish_approval',
      to: 'workspaces#publish_approval', defaults: {format: :json}

    put 'users/mget' => 'users#mget', defaults: {format: :json}
    get 'users/suggest' => 'users#suggest', defaults: {format: :json}

    put 'ui_configs/mget' => 'ui_configs#mget', defaults: {format: :json}
    put 'publish_abilities/mget' => 'publish_abilities#mget', defaults: {format: :json}

    get 'render_widget/:id/show_widget/:widget_id' => 'cms_dispatch#show_widget'
    get 'render_widget/:id/widget_details/:widget_id' => 'cms_dispatch#widget_details'

    get 'page_details/:id' => 'cms_dispatch#page_details'
    get 'resource_details/:resource_id', to: 'ui#index'

    get 'to_binary' => 'binary_redirect#to_binary', as: :binary
  end
end
